@@ -1,6 +1,6 @@
 # This is a sample hardware file for UDP control.  Use this file for my 2010 transceiver
 # described in QEX and for the improved version HiQSDR.  To turn on the extended
-# features in HiQSDR, update your FPGA firmware to version 1.1 and use use_rx_udp = 2.
+# features in HiQSDR, update your FPGA firmware to version 1.1 or later and use use_rx_udp = 2.
 
 import struct, socket
 import _quisk as QS
@@ -55,6 +55,7 @@ class Hardware(BaseHardware):
     self.vna_count = 0	# VNA scan count; MUST be zero for non-VNA operation
     self.index = 0
     self.mode = None
+    self.band = None
     self.HiQSDR_Connector_X1 = 0
     self.HiQSDR_Attenuator = 0
     self.HiQSDR_Bits = 0
@@ -101,20 +102,29 @@ class Hardware(BaseHardware):
       pass
     elif mode in ("CWL", "CWU"):
       self.tx_control |= 0x01
-    elif mode in ("USB", "LSB", "AM", "FM"):
+    elif mode in ("USB", "LSB", "AM", "FM", "DGTL"):
       self.tx_control |= 0x02
     elif mode[0:3] == 'IMD':
       self.tx_control |= 0x02
+    self.SetTxLevel()
     self.NewUdpStatus()
   def ChangeBand(self, band):
     # band is a string: "60", "40", "WWV", etc.
-    try:
-      self.tx_level = self.conf.tx_level[band]
-    except KeyError:
-      self.tx_level = self.conf.tx_level[None]		# The default
+    self.band = band
     self.HiQSDR_Connector_X1 &= ~0x0F	# Mask in the last four bits
     self.HiQSDR_Connector_X1 |= self.conf.HiQSDR_BandDict.get(band, 0) & 0x0F
+    self.SetTxLevel()
     self.NewUdpStatus()
+  def SetTxLevel(self):
+    if self.vna_count:
+      return
+    elif self.mode == 'DGTL':
+      self.tx_level = self.conf.digital_tx_level
+    else:
+      try:
+        self.tx_level = self.conf.tx_level[self.band]
+      except KeyError:
+        self.tx_level = self.conf.tx_level[None]		# The default
   def OnButtonRfGain(self, event):
     # The HiQSDR attenuator is five bits: 2, 4, 8, 10, 20 dB
     btn = event.GetEventObject()
