@@ -28,7 +28,7 @@ static int debug_timer = 1;		// count up number of samples
 // FM needs pre-emphasis and de-emphasis.  See vk1od.net/FM/FM.htm for details.
 // For IIR design, see http://www.abvolt.com/research/publications2.htm.
 
-// Microhone preemphasis: boost high frequencies 0.00 to 1.00; or -1.0 for Butterworth
+// Microhone preemphasis: boost high frequencies 0.00 to 1.00
 double quisk_mic_preemphasis;
 // Microphone clipping; try 3.0 or 4.0
 double quisk_mic_clip;
@@ -350,6 +350,61 @@ static int tx_filter(complex * filtered, int count, int is_cpx)
 	return count;
 }
 
+static int tx_filter_digital(complex * filtered, int count)
+{	// Input samples are creal(filtered), output is filtered.
+	// This filter has minimal processing and is used for digital modes.
+	int i;
+	double dsample, amplitude;
+	complex csample;
+
+	static struct quisk_dFilter filter1;
+	static int do_init = 1;
+#if DEBUG_IO
+	double x;
+	static double peakIn = 0, peakOut2 = 0;		// input/output level
+#endif
+	if (do_init) {		// initialization
+		do_init = 0;
+		quisk_filt_dInit(&filter1, quiskMicFilt48Coefs, sizeof(quiskMicFilt48Coefs)/sizeof(double));
+		quisk_filt_tune(&filter1, 1650.0 / 48000, 1);
+	}
+#if DEBUG_IO
+	//QuiskPrintTime("", -2);
+#endif
+	for (i = 0; i < count; i++) {
+		dsample = creal(filtered[i]) / CLIP16;		// normalize to +/- 1.0
+#if DEBUG_IO
+		x = fabs(dsample);
+		if (x > peakIn)
+			peakIn = x;
+#endif
+		// FIR bandpass filter; separate into I and Q
+		csample = quisk_dC_out(dsample, &filter1);
+		amplitude = cabs(csample);
+#if DEBUG_IO
+		if (amplitude > peakOut2)
+			peakOut2 = amplitude;
+#endif
+		if (amplitude > 1.0)
+			csample /= amplitude;
+		filtered[i] = csample * CLIP16;		// convert back to 16 bits
+	}
+
+//printf("%5.2lf\n", increasing);
+#if DEBUG_IO
+	if (debug_timer == 0) {
+		printf ("peakIn %10.6lf  peakOut2 %10.6lf", peakIn, peakOut2);
+		if (peakOut2 > 1.0)
+			printf ("  CLIP\n");
+		else
+			printf ("\n");
+		peakIn = peakOut2 = 0;
+	}
+	//QuiskPrintTime("    tx_filter", 2);
+#endif
+	return count;
+}
+
 PyObject * quisk_get_tx_filter(PyObject * self, PyObject * args)
 {  // return the TX filter response to display on the graph
 // This is for debugging.  Change quisk.py to call QS.get_tx_filter() instead
@@ -450,7 +505,6 @@ static void transmit_udp(complex * cSamples, int count)
 	int i, sent;
 	static short udp_iq[TX_BLOCK_SHORTS + 1] = {0};
 	static int udp_size = 1;
-	double vol;
 
 	if (mic_socket == INVALID_SOCKET)
 		return;
@@ -459,10 +513,9 @@ static void transmit_udp(complex * cSamples, int count)
 		udp_iq[0] = 0;	// should not be necessary
 		return;
 	}
-	vol = quisk_sound_state.mic_out_volume;
 	for (i = 0; i < count; i++) {	// transmit samples
-		udp_iq[udp_size++] = (short)(creal(cSamples[i]) * vol);
-		udp_iq[udp_size++] = (short)(cimag(cSamples[i]) * vol);
+		udp_iq[udp_size++] = (short)creal(cSamples[i]);
+		udp_iq[udp_size++] = (short)cimag(cSamples[i]);
 		if (udp_size >= TX_BLOCK_SHORTS) {	// check count
 			if (align4)
 				sent = send(mic_socket, (char *)udp_iq, udp_size * 2, 0);
@@ -507,14 +560,14 @@ static void transmit_mic_imd(complex * cSamples, int count, double level)
 	transmit_udp(cSamples, count);
 }
 
-int quisk_process_microphone(complex * cSamples, int count)
+int quisk_process_microphone(int mic_sample_rate, complex * cSamples, int count)
 {
 	int i, sample, maximum, interp;
 	double d;
 
-// Microphone sample are input at quisk_sound_state.mic_sample_rate.  But after processing,
+// Microphone sample are input at mic_sample_rate.  But after processing,
 // the output rate is MIC_OUT_RATE.
-	interp = MIC_OUT_RATE / quisk_sound_state.mic_sample_rate;
+	interp = MIC_OUT_RATE / mic_sample_rate;
 
 #if 0
 	// Measure soundcard actual sample rate
@@ -542,7 +595,7 @@ int quisk_process_microphone(complex * cSamples, int count)
 
 #if DEBUG_IO
 	debug_timer += count;
-	if (debug_timer >= quisk_sound_state.mic_sample_rate)		// one second
+	if (debug_timer >= mic_sample_rate)		// one second
 		debug_timer = 0;
 #endif
 	
@@ -567,7 +620,7 @@ int quisk_process_microphone(complex * cSamples, int count)
 		mic_level = maximum;
 	mic_timer -= count;		// time out the max microphone level to display
 	if (mic_timer <= 0) {
-		mic_timer = quisk_sound_state.mic_sample_rate / 1000 * MIC_MAX_HOLD_TIME;
+		mic_timer = mic_sample_rate / 1000 * MIC_MAX_HOLD_TIME;
 		mic_max_display = mic_level;
 		mic_level = 1;
 	}
@@ -601,6 +654,16 @@ int quisk_process_microphone(complex * cSamples, int count)
 			}
   			transmit_udp(cSamples, count);
   			break;
+		case 7:		// DGTL
+			if (spotLevel == 0) {
+				count = tx_filter_digital(cSamples, count);	// filter samples, minimal processing
+				transmit_udp(cSamples, count);
+			}
+			else {
+				count *= interp;
+				transmit_mic_carrier(cSamples, count, spotLevel / 1000.0);
+			}
+  			break;
 		case 10:	// transmit IMD 2-tone test
 			count *= interp;
 			transmit_mic_imd(cSamples, count, 1.0);
