@@ -10,9 +10,10 @@ if sys.platform == "win32":
 else:
   name_of_sound_capt = 'hw:0'  #'alsa:Audiophile'
   name_of_sound_play = 'hw:0'    #"alsa:USB Audio CODEC"
-  latency_millisecs = 50
+  latency_millisecs = 150
   data_poll_usec = 5000
 
+sdriq_name = "/dev/ttyUSB0"		# Name of the SDR-IQ device to open
 mic_clip = 3.0
 mic_preemphasis = 0.6
 
@@ -22,6 +23,7 @@ waterfall_y_zero  = 40
 waterfall_graph_y_scale = 40
 waterfall_graph_y_zero = 90
 waterfall_graph_size = 160
+display_fraction = 1.00			# The edges of the full bandwidth are not valid
 
 if microphone_name:
   mixer_settings = [
