@@ -13,6 +13,13 @@ if sys.platform == "win32":
   name_of_mic_play = "Primary"
   latency_millisecs = 150
   data_poll_usec = 10000
+elif 1:
+  name_of_sound_capt = 'portaudio:Analog'
+  name_of_sound_play = ''
+  microphone_name = "portaudio:AK5370"
+  name_of_mic_play = 'portaudio:Analog'
+  latency_millisecs = 150
+  data_poll_usec = 5000
 else:
   name_of_sound_capt = 'hw:0'
   name_of_sound_play = ''
@@ -21,9 +28,11 @@ else:
   latency_millisecs = 50
   data_poll_usec = 5000
 
+add_fdx_button = 1
 sample_rate = 48000
 mic_playback_rate = 48000
-mic_out_volume = 0.6
+mic_sample_rate = 48000
+mic_out_volume = 0.7
 
 mic_clip = 3.0
 mic_preemphasis = 0.6
