@@ -18,36 +18,12 @@
 #endif
 
 #include "quisk.h"
+#include "filter.h"
 
 #define DEBUG		0
 
 #define FM_FILTER_DEMPH		300.0		// Frequency of FM lowpass de-emphasis filter
 
-// Interpolation filter
-double interpFilterCoef[INTERP_FILTER_TAPS] = {
--0.0014520724581619038, 0.001465077835702283, 0.0016872049554487805, 0.002231589757636616,
-0.002968733413841478, 0.003814111604268629, 0.004701377925622313, 0.005563735642891225,
-0.006329349313192236, 0.00692563503945522, 0.00728031065507179, 0.007324031007188439,
-0.007001538274824099, 0.006269549743028705, 0.005109362626336052, 0.0035251497078111612,
-0.0015532783824581805, -7.42007695505918E-4, -0.003261456038073849, -0.00588052913811741,
--0.008445715457113093, -0.010788117896010696, -0.012724127539638374, -0.014072167012043064,
--0.014655432422717352, -0.014320398913272279, -0.012939708244425117, -0.010428725793659156,
--0.00674649422603162, -0.001908771663085817, 0.004016522608274192, 0.010900987627517972,
-0.018565817817064456, 0.026782245665652966, 0.03528488587061357, 0.04377887008351154,
-0.05195918785452774, 0.05951705401787405, 0.06616030228971032, 0.0716318254603302,
-0.07570134752536944, 0.07821336538322851, 0.07906313690411886, 0.07821336538322851,
-0.07570134752536944, 0.0716318254603302, 0.06616030228971032, 0.05951705401787405,
-0.05195918785452774, 0.04377887008351154, 0.03528488587061357, 0.026782245665652966,
-0.018565817817064456, 0.010900987627517972, 0.004016522608274192, -0.001908771663085817,
--0.00674649422603162, -0.010428725793659156, -0.012939708244425117, -0.014320398913272279,
--0.014655432422717352, -0.014072167012043064, -0.012724127539638374, -0.010788117896010696,
--0.008445715457113093, -0.00588052913811741, -0.003261456038073849, -7.42007695505918E-4,
-0.0015532783824581805, 0.0035251497078111612, 0.005109362626336052, 0.006269549743028705,
-0.007001538274824099, 0.007324031007188439, 0.00728031065507179, 0.00692563503945522,
-0.006329349313192236, 0.005563735642891225, 0.004701377925622313, 0.003814111604268629,
-0.002968733413841478, 0.002231589757636616, 0.0016872049554487805, 0.001465077835702283,
--0.0014520724581619038};
-
 static int fft_error;			// fft error count
 static int count_fft;			// how many fft's have occurred (for average)
 enum fft_status {EMPTY,			// fft_data is currently unused
@@ -69,8 +45,8 @@ static int use_remove_dc=1;		// Remove DC from samples
 static PyObject * QuiskError;		// Exception for this module
 static PyObject * pyApp;		// Application instance
 static int fft_size;			// size of fft, e.g. 1024
-int data_width;					// number of points to return as graph data; fft_size * n
-int rxMode;						// 0 to 6, 10 to 12: CWL, CWU, LSB, USB, AM, FM, EXT, IMD(10 to 12)
+int data_width;				// number of points to return as graph data; fft_size * n
+int rxMode;				// 0 to 7, 10 to 12: CWL, CWU, LSB, USB, AM, FM, EXT, DGTL, IMD(10 to 12)
 int quisk_noise_blanker;		// noise blanker level, 0 for off
 PyObject * quisk_pyConfig=NULL;	// Configuration module instance
 long quisk_mainwin_handle;		// Handle of the main window
@@ -92,14 +68,10 @@ static int indexFilter;			// Index of current filter data in buffer
 static int isFDX;			// Are we in full duplex mode?
 static int filter_bandwidth;		// Current filter bandwidth in Hertz
 
-static double * fmFilterI;		// FM audio filter
-static double * fmFilterBufI;
-static int fmFilterSize = 0;
-
 static double sidetoneVolume;		// Audio output level of the CW sidetone, 0.0 to 1.0
 static int keyupDelay;			// Play silence after sidetone ends
 static complex sidetonePhase;		// Phase increment for sidetone
-int quisk_sidetoneCtrl;			// sidetone control value
+int quisk_sidetoneCtrl;			// sidetone control value 0 to 1000
 
 static int agcInUse;			// 0 for no AGC
 static double agcRelease, agcLevel;	// AGC parameters
@@ -123,70 +95,49 @@ static int rx_udp_read_blocks = 0;	// Number of blocks to read for each read cal
 
 static int is_little_endian;		// Test byte order; is it little-endian?
 
-
 static int fDecimate(double * dSamples, int nSamples, double fdecim)
-{  // Decimate to a lower sample rate by a fraction
-#if 1
-		// fractional decimation with linear interpolation
-		int n, nout, avail;
-		static double dindex = 0;
-		static double lastsample = 0;
-		double slast, samp, s1, s2;
+{
+	int i, nout;
+	double xm0, xm1, xm2, xm3;
+	static double dindex = 1;
+	static double y0=0, y1=0, y2=0, y3=0;
+	static int in=0, out=0;
 
-		slast = dSamples[nSamples - 1];		// save last sample
-		if (dindex < 0) {	// use last sample
-			n = -1;
-			s1 = lastsample;
-			s2 = dSamples[0];
-			samp = s1 * (1.0 - dindex + n) + s2 * (dindex - n);
-			avail = 1;
-			dindex += fdecim;
+	in += nSamples;
+	nout = 0;
+	for (i = 0; i < nSamples; i++) {
+		y3 = dSamples[i];
+		if (dindex < 1 || dindex >= 2.2)
+			printf ("dindex %.5f  fdecim %.8f\n", dindex, fdecim);
+		if (dindex < 2) {
+#if 0
+				dSamples[nout++] = (1 - (dindex - 1)) * y1 + (dindex - 1) * y2;
+#else
+				xm0 = dindex - 0;
+				xm1 = dindex - 1;
+				xm2 = dindex - 2;
+				xm3 = dindex - 3;
+				dSamples[nout++] = xm1 * xm2 * xm3 * y0 / -6.0 + xm0 * xm2 * xm3 * y1 / 2.0 +
+					xm0 * xm1 * xm3 * y2 / -2.0 + xm0 * xm1 * xm2 * y3 / 6.0;
+#endif
+			out++;
+			dindex += fdecim - 1;
+			y0 = y1;
+			y1 = y2;
+			y2 = y3;
 		}
 		else {
-			avail = 0;
-			samp = 0;
+			if (dindex > 2.3) printf ("Skip at %.2f\n", dindex);
+			y0 = y1;
+			y1 = y2;
+			y2 = y3;
+			dindex -= 1;
 		}
-		nout = 0;
-		n = (int)(floor(dindex) + 0.1);
-		// Calculate any subsequent samples
-		while (n < nSamples - 1) {
-			s1 = dSamples[n];
-			s2 = dSamples[n + 1];
-			if (avail) {		// record delayed sample
-				dSamples[nout++] = samp;
-			}
-			samp = s1 * (1.0 - dindex + n) + s2 * (dindex - n);
-			avail = 1;
-			dindex += fdecim;
-			n = (int)(floor(dindex) + 0.1);
-		}
-		if (avail) {
-			dSamples[nout++] = samp;	// record last sample
-		}
-		lastsample = slast;
-		dindex -= nSamples;
-		//if (dindex < -1)
-		//	printf("dindex %.2f, i %d\n", dindex, i);
-		return nout;
-#else
-		// fractional decimation with nearest neighbor
-		int n, nout;
-		static double dindex = 0;
-if (! dSamples) {dindex = 0; return 0;}
-		n = (int)(dindex + 0.5);
-		for (nout = 0; n < nSamples; ) {
-			dSamples[nout++] = dSamples[n];
-if (prnt) printf ("fD  %d %d %.3f  %.3f\n", nout-1, n, dindex, dSamples[n]);
-			dindex += fdecim;
-			n = (int)(dindex + 0.5);
-		}
-		dindex -= nSamples;
-		//if (dindex + 0.5 < 0)
-		//	printf("dindex %.2f, i %d\n", dindex, i);
-		return nout;
-#endif
+	}
+	//printf ("in %d out %d\n", in, out);
+	return nout;
 }
-
+			
 #define QUISK_NB_HWINDOW_SECS	500.E-6	// half-size of blanking window in seconds
 static void NoiseBlanker(complex * cSamples, int nSamples)
 {
@@ -294,67 +245,95 @@ static void NoiseBlanker(complex * cSamples, int nSamples)
 	return;
 }
 
-#if 0
-/* Data for the * 9 / 25 decimation filter */
-#define SDRIQ_NTAPS		36			// Must be a multiple of 9
-//Parks-McClellan FIR Filter Design
-//http://www.dsptutor.freeuk.com/remez/RemezFIRFilterDesign.html
-//Filter type: Low pass
-//Passband: 0 - 0.005
-//Order: 35
-//Passband ripple: 0.1 dB
-//Transition band: 0.06
-//Stopband attenuation: 80.0 dB [actual 58 dB]
-static double sdriq_coef[SDRIQ_NTAPS] = {		// 0.005 bandwidth, 0.06 transition
-0.0014619269216906926, 0.002107635096689865, 0.0034965759202008955, 0.00535035563967141,
-0.007715621632504707, 0.010606609406202767, 0.01401419424317856, 0.01789312723553899,
-0.022165821144938178, 0.02671891463904301, 0.03141161349009851, 0.0360793011043149,
-0.04054288348471436, 0.04461756403666611, 0.04812662001443451, 0.050912445136915334,
-0.05284675928590912, 0.053837896731515664, 0.053837896731515664, 0.05284675928590912,
-0.050912445136915334, 0.04812662001443451, 0.04461756403666611, 0.04054288348471436,
-0.0360793011043149, 0.03141161349009851, 0.02671891463904301, 0.022165821144938178,
-0.01789312723553899, 0.01401419424317856, 0.010606609406202767, 0.007715621632504707,
-0.00535035563967141, 0.0034965759202008955, 0.002107635096689865, 0.0014619269216906926} ;
+static complex dRxFilterOut(complex sample)
+{
+	complex cx;
+	int j, k;
 
-static int sdrDecimate(complex * cSamples, int nSamples, int fsize, double * coef)
-{	// Interpolate by 9, filter, decimate by 25.
-	int i, j, k, nout;
-	static int iavail = 0;		// index of first available sample
-	static complex delay[MAX_FILTER_SIZE] = {0};		// saved prior samples
-	complex sample, out;
+	if ( ! sizeFilter)
+		return sample;
+	bufFilterC[indexFilter] = sample;
+	cx = 0;
+	j = indexFilter;
+	for (k = 0; k < sizeFilter; k++) {
+		cx += bufFilterC[j] * cFilterI[k];
+		if (++j >= sizeFilter)
+			j = 0;
+	}
+	if (++indexFilter >= sizeFilter)
+		indexFilter = 0;
+	return cx;
+}
 
-	nout = 0;
-	for (i = 0; i < nSamples; i++) {
-		sample = cSamples[i];
-		iavail += 9;		// nine more samples are available
-		if (iavail > 15) {
-			k = 24 - iavail;	// output the k'th sample; k is 0 thru 8
-			iavail = -k - 1;
-			out = sample * coef[k];
-			j = k * (fsize / 9 - 1);
-			for (k += 9; k <  fsize; k += 9, j++)
-				out += coef[k] * delay[j];
-			cSamples[nout++] = out * 9.0;	// Interpolation loss is 1 / 9
+static complex cRxFilterOut(complex sample)
+{
+	double accI, accQ;
+	int j, k;
+
+	if ( ! sizeFilter)
+		return sample;
+	bufFilterI[indexFilter] = creal(sample);
+	bufFilterQ[indexFilter] = cimag(sample);
+	accI = accQ = 0;
+	j = indexFilter;
+	for (k = 0; k < sizeFilter; k++) {
+		accI += bufFilterI[j] * cFilterI[k];
+		accQ += bufFilterQ[j] * cFilterQ[k];
+		if (++j >= sizeFilter)
+			j = 0;
+	}
+	if (++indexFilter >= sizeFilter)
+		indexFilter = 0;
+	return accI + I * accQ;
+}
+
+static void AddTestTone(complex * cSamples, int nSamples)
+{
+	int i;
+	static complex testtoneVector = 21474836.47;	// -40 dB
+	static complex audioVector = 1.0;
+	complex audioPhase;
+
+	switch (rxMode) {
+	default:
+		//testtonePhase = cexp(I * 2 * M_PI * (quisk_sidetoneCtrl - 500) / 1000.0);
+		for (i = 0; i < nSamples; i++) {
+			cSamples[i] += testtoneVector;
+			testtoneVector *= testtonePhase;
+		}
+		break;
+	case 4:		// AM
+		//audioPhase = cexp(I * 2 * M_PI * quisk_sidetoneCtrl * 5 / sample_rate);
+		audioPhase = cexp(I * 2.0 * M_PI * 1000 / sample_rate);
+		for (i = 0; i < nSamples; i++) {
+			cSamples[i] += testtoneVector * (1.0 + creal(audioVector));
+			testtoneVector *= testtonePhase;
+			audioVector *= audioPhase;
+		}
+		break;
+	case 5:		 // FM
+		//audioPhase = cexp(I * 2 * M_PI * quisk_sidetoneCtrl * 5 / sample_rate);
+		audioPhase = cexp(I * 2.0 * M_PI * 1000 / sample_rate);
+		for (i = 0; i < nSamples; i++) {
+			cSamples[i] += testtoneVector * cexp(I * creal(audioVector));
+			testtoneVector *= testtonePhase;
+			audioVector *= audioPhase;
 		}
-		// Move down delayed samples, record current sample as first delay
-		memmove(delay + 1, delay, sizeof(complex) * (fsize - 9 - 1));
-		for (j = 0; j < fsize - 9; j += fsize / 9 - 1)
-			delay[j] = sample;
+		break;
 	}
-	return nout;
 }
-#endif
 
 int quisk_process_samples(complex * cSamples, int nSamples)
 {
 // Called when samples are available.
 // Samples range from about 2^16 to a max of 2^31.
-	int i, j, k, nout, filter_srate, is_key_down;
-	double d, di, accI, accQ, agc_level, agcPeak, www, nnn, a_0, a_1, b_1;
+	int i, nout, filter_srate, is_key_down, interp, final_filter;
+	double d, di, agc_level, agcPeak;
+	double double_filter_decim;
 	complex cx, cpx;
 	double dsamples[SAMP_BUFFER_SIZE];
 
-	static complex testtoneVector = 21474836.47;	// -40 dB
+	static double FM_www, FM_nnn, FM_a_0, FM_a_1, FM_b_1, FM_x_1, FM_y_1;   // filter for FM
 	static complex tuneVector = 1;
 	static complex sidetoneVector = BIG_VOLUME;
 	static double dOutCounter = 0;		// Cumulative net output samples for sidetone etc.
@@ -363,15 +342,32 @@ int quisk_process_samples(complex * cSamples, int nSamples)
 	static double keyupEnvelope = 1.0;	// Shape the rise time on key up
 	static int playSilence;
 	static complex fm_1 = 10;		// Sample delayed by one
-	static int indexFmFilter = 0;		// index into FM filter
-	static int indexInterpFilter = 0;	// index into interpolation filter
-	static double interpFilterBuf[INTERP_FILTER_TAPS];		// Interpolation audio filter
-	static double x_1, y_1;				// FM de-emphasis filter
 	static double dc_remove;		// DC removal for AM
+	static struct quisk_cHB45Filter HalfBand1 = {NULL, 0, 0};
+	static struct quisk_cHB45Filter HalfBand2 = {NULL, 0, 0};
+	static struct quisk_cHB45Filter HalfBand3 = {NULL, 0, 0};
+	static struct quisk_cHB45Filter HalfBand4 = {NULL, 0, 0};
+	static struct quisk_cHB45Filter HalfBand5 = {NULL, 0, 0};
+	static struct quisk_dHB45Filter HalfBand6 = {NULL, 0, 0};
+	static struct quisk_dHB45Filter HalfBand7 = {NULL, 0, 0};
+	static struct quisk_dHB45Filter HalfBand8 = {NULL, 0, 0};
+	static struct quisk_dHB45Filter HalfBand9 = {NULL, 0, 0};
+	static struct quisk_cFilter filtSdriq2 = {NULL};
+	static struct quisk_cFilter filtSdriq4;
+	static struct quisk_cFilter filtSdriq5;
+	static struct quisk_cFilter filtSdriq7;
+	static struct quisk_cFilter filtDecim5;
+	static struct quisk_cFilter filtDecim48to24;
+	static struct quisk_dFilter filtAudio24p3;
+	static struct quisk_dFilter filtAudio24p4;
+	static struct quisk_dFilter filtAudio12p2;
+	static struct quisk_dFilter filtAudio24p6;
+	static struct quisk_dFilter filtAudioFmHp;
 
 #if DEBUG
 	static int printit;
 	static time_t time0;
+	static double levelA=0, levelB=0, levelC=0, levelD=0, levelE=0;
 
 	if (time(NULL) != time0) {
 		time0 = time(NULL);
@@ -380,10 +376,26 @@ int quisk_process_samples(complex * cSamples, int nSamples)
 	else {
 		printit = 0;
 	}
-
-	//if (quisk_sound_state.interupts < 10 || quisk_sound_state.interupts % 1000 == 0)
-	//printf("Interupt %d\n", quisk_sound_state.interupts);
 #endif
+	if ( ! filtSdriq2.dCoefs) {	// Initialize all filters
+		quisk_filt_cInit(&filtSdriq2, quiskFilt53D2Coefs, sizeof(quiskFilt53D2Coefs)/sizeof(double));
+		quisk_filt_cInit(&filtSdriq4, quiskFilt111D4Coefs, sizeof(quiskFilt111D4Coefs)/sizeof(double));
+		quisk_filt_cInit(&filtSdriq5, quiskFilt133D5Coefs, sizeof(quiskFilt133D5Coefs)/sizeof(double));
+		quisk_filt_cInit(&filtSdriq7, quiskFilt185D7Coefs, sizeof(quiskFilt185D7Coefs)/sizeof(double));
+		quisk_filt_cInit(&filtDecim5, quiskFilt240D5Coefs, sizeof(quiskFilt240D5Coefs)/sizeof(double));
+		quisk_filt_cInit(&filtDecim48to24, quiskFilt48dec24Coefs, sizeof(quiskFilt48dec24Coefs)/sizeof(double));
+		quisk_filt_dInit(&filtAudio24p3,  quiskAudio24p3Coefs, sizeof(quiskAudio24p3Coefs)/sizeof(double));
+		quisk_filt_dInit(&filtAudio24p4,  quiskAudio24p4Coefs, sizeof(quiskAudio24p4Coefs)/sizeof(double));
+		quisk_filt_dInit(&filtAudio12p2, quiskAudio24p4Coefs, sizeof(quiskAudio24p4Coefs)/sizeof(double));
+		quisk_filt_dInit(&filtAudio24p6, quiskAudio24p6Coefs, sizeof(quiskAudio24p6Coefs)/sizeof(double));
+		quisk_filt_dInit(&filtAudioFmHp, quiskAudioFmHpCoefs, sizeof(quiskAudioFmHpCoefs)/sizeof(double));
+		FM_www = tan(M_PI * FM_FILTER_DEMPH / 24000);   // filter for FM
+		FM_nnn = 1.0 / (1.0 + FM_www);
+		FM_a_0 = FM_www * FM_nnn;
+		FM_a_1 = FM_a_0;
+		FM_b_1 = FM_nnn * (FM_www - 1.0);
+		//printf ("dsamples[i] = y_1 = di * %12.6lf + x_1 * %12.6lf - y_1 * %12.6lf\n", FM_a_0, FM_a_1, FM_b_1);
+	}
 	if (nSamples <= 0)
 		return nSamples;
 	is_key_down = quisk_is_key_down();
@@ -455,12 +467,9 @@ int quisk_process_samples(complex * cSamples, int nSamples)
 	// demodulate the samples as radio sound.
 
 	// Add a test tone to the data
-	if (testtonePhase) {
-		for (i = 0; i < nSamples; i++) {
-			cSamples[i] += testtoneVector;
-			testtoneVector *= testtonePhase;
-		}
-	}
+	if (testtonePhase)
+		AddTestTone(cSamples, nSamples);
+
 	// Invert spectrum
 	if (quisk_invert_spectrum) {
 		for (i = 0; i < nSamples; i++) {
@@ -506,8 +515,7 @@ int quisk_process_samples(complex * cSamples, int nSamples)
 	}
 
 	if (rxMode == 6) {		// External filter and demodulate
-		d = quisk_sound_state.double_filter_decim * quisk_sound_state.int_filter_decim /
-				quisk_sound_state.int_filter_interp;	// total decimation needed
+		d = (double)quisk_sound_state.sample_rate / quisk_sound_state.playback_rate;	// total decimation needed
 		nSamples = quisk_extern_demod(cSamples, nSamples, d);
 		// Find the peak signal amplitude
 		agcPeak = 0;
@@ -535,133 +543,214 @@ int quisk_process_samples(complex * cSamples, int nSamples)
 			cSamples[i] = cimag(cSamples[i]) * di;
 		return nSamples;
 	}
-
-	// Decimate: Lower the sample rate.
-	nSamples = quisk_iDecimate (cSamples, nSamples, quisk_sound_state.int_filter_decim);
-
-	/* Filter the signal */
-	if (sizeFilter) {
-		if (rxMode == 4 || rxMode == 5)	{	// AM and FM: Use same filter for I and Q
-			for (i = 0; i < nSamples; i++) {
-				bufFilterC[indexFilter] = cSamples[i];
-				cx = 0;
-				j = indexFilter;
-				for (k = 0; k < sizeFilter; k++) {
-					cx += bufFilterC[j] * cFilterI[k];
-					if (++j >= sizeFilter)
-						j = 0;
-				}
-				cSamples[i] = cx;
-				if (++indexFilter >= sizeFilter)
-					indexFilter = 0;
-			}
-		}
-		else {			// CW and SSB: Use filters for 90 degree phase shift
-			for (i = 0; i < nSamples; i++) {
-				cx = cSamples[i];
-				bufFilterI[indexFilter] = creal(cx);
-				bufFilterQ[indexFilter] = cimag(cx);
-				accI = accQ = 0;
-				j = indexFilter;
-				for (k = 0; k < sizeFilter; k++) {
-					accI += bufFilterI[j] * cFilterI[k];
-					accQ += bufFilterQ[j] * cFilterQ[k];
-					if (++j >= sizeFilter)
-						j = 0;
-				}
-				cSamples[i] = accI + I * accQ;
-				if (++indexFilter >= sizeFilter)
-					indexFilter = 0;
-			}
-		}
-	}
-	// Demodulate signal, copy capture buffer cSamples to play buffer dsamples.
-	// filter_srate is the sample rate after decimation.
-	filter_srate = quisk_sound_state.sample_rate / quisk_sound_state.int_filter_decim;
+#if DEBUG
 	for (i = 0; i < nSamples; i++) {
-		cx = cSamples[i];
-		switch(rxMode) {
-		case 0:		// lower sideband
-		case 2:
-			di = creal(cx) + cimag(cx);
-			break;
-		case 1:		// upper sideband
-		case 3:
-		default:
-			di = creal(cx) - cimag(cx);
-			break;
-		case 4:		// AM
+		d = cabs(cSamples[i]);
+		if (levelA < d)
+			levelA = d;
+	}
+#endif
+
+	// Decimate: Lower the sample rate to 24000 sps (or approx).  Filters are designed for
+	// a pass bandwidth of 10 kHz and a stop bandwidth of 12 kHz.
+	final_filter = rxMode == 4 || rxMode == 5;	// Use sharp FIR final filter for decimate by 2
+	double_filter_decim = 1.0;
+	switch((quisk_sound_state.sample_rate + 100) / 1000) {
+	case 24:
+		break;
+	case 41:
+	case 48:
+		if (final_filter)
+			nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		else
+			nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand5);
+		break;
+	case 53:	// SDR-IQ
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtSdriq2, 2);
+		double_filter_decim = quisk_sound_state.sample_rate / 2.0 / 24000.0;
+		break;
+	case 96:
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand1);
+		if (final_filter)
+			nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		else
+			nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand5);
+		break;
+	case 111:	// SDR-IQ
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtSdriq4, 4);
+		double_filter_decim = quisk_sound_state.sample_rate / 4.0 / 24000.0;
+		break;
+	case 133:	// SDR-IQ
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtSdriq5, 5);
+		double_filter_decim = quisk_sound_state.sample_rate / 5.0 / 24000.0;
+		break;
+	case 185:	// SDR-IQ
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtSdriq7, 7);
+		double_filter_decim = quisk_sound_state.sample_rate / 7.0 / 24000.0;
+		break;
+	case 192:
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand1);
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand2);
+		if (final_filter)
+			nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		else
+			nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand5);
+		break;
+	case 240:
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim5, 5);
+		if (final_filter)
+			nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		else
+			nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand5);
+		break;
+	case 384:
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand1);
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand2);
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand3);
+		if (final_filter)
+			nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		else
+			nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand5);
+		break;
+	case 480:
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand1);
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim5, 5);
+		if (final_filter)
+			nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		else
+			nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand5);
+		break;
+	case 960:
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand1);
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand2);
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim5, 5);
+		if (final_filter)
+			nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		else
+			nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand5);
+		break;
+	default:
+		printf ("Failure in quisk.c in integer decimation\n");
+		break;
+	}
+
+#if DEBUG
+	for (i = 0; i < nSamples; i++) {
+		d = cabs(cSamples[i]);
+		if (levelB < d)
+			levelB = d;
+	}
+#endif
+
+	// Filter and demodulate signal, copy capture buffer cSamples to play buffer dsamples.
+	// filter_srate is the sample rate after integer decimation.
+	filter_srate = 24000;
+	switch(rxMode) {
+	case 0:		// lower sideband CW at 6 ksps
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand4);
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		for (i = 0; i < nSamples; i++) {
+			cx = cRxFilterOut(cSamples[i]);
+			dsamples[i] = creal(cx) + cimag(cx);
+		}
+		nSamples = quisk_dInterpolate(dsamples, nSamples, &filtAudio12p2, 2);
+		nSamples = quisk_dInterp2HB45(dsamples, nSamples, &HalfBand6);
+		break;
+	case 1:		// upper sideband CW at 6 ksps
+		nSamples = quisk_cDecim2HB45(cSamples, nSamples, &HalfBand4);
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		for (i = 0; i < nSamples; i++) {
+			cx = cRxFilterOut(cSamples[i]);
+			dsamples[i] = creal(cx) - cimag(cx);
+		}
+		nSamples = quisk_dInterpolate(dsamples, nSamples, &filtAudio12p2, 2);
+		nSamples = quisk_dInterp2HB45(dsamples, nSamples, &HalfBand6);
+		break;
+	case 2:	 // lower sideband SSB at 12 ksps
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		for (i = 0; i < nSamples; i++) {
+			cx = cRxFilterOut(cSamples[i]);
+			dsamples[i] = creal(cx) + cimag(cx);
+		}
+		nSamples = quisk_dInterpolate(dsamples, nSamples, &filtAudio24p4, 2);
+		break;
+	case 3:	 // upper sideband SSB at 12 ksps
+	case 7:
+	default:
+		nSamples = quisk_cDecimate(cSamples, nSamples, &filtDecim48to24, 2);
+		for (i = 0; i < nSamples; i++) {
+			cx = cRxFilterOut(cSamples[i]);
+			dsamples[i] = creal(cx) - cimag(cx);
+		}
+		nSamples = quisk_dInterpolate(dsamples, nSamples, &filtAudio24p4, 2);
+		break;
+	case 4:		// AM at 24 ksps
+		for (i = 0; i < nSamples; i++) {
+			cx = dRxFilterOut(cSamples[i]);
 			di = cabs(cx);
-			break;
-		case 5:		// FM
+			d = di + dc_remove * 0.99;	// DC removal; R.G. Lyons page 553
+			di = d - dc_remove;
+			dc_remove = d;
+			dsamples[i] = di;
+		}
+		nSamples = quisk_dFilter(dsamples, nSamples, &filtAudio24p6);
+		break;
+	case 5:		// FM at 24 ksps
+		for (i = 0; i < nSamples; i++) {
+			cx = dRxFilterOut(cSamples[i]);
 			cpx = cx * conj(fm_1);
 			fm_1 = cx;
 			di = filter_srate * carg(cpx);
-			break;
-		}
-		dsamples[i] = di;
-	}
-	// For AM and FM, filter the audio
-	if (rxMode == 4 || rxMode == 5) {
-		for (i = 0; i < nSamples; i++) {	// DC removal; R.G. Lyons page 553
-			d = dsamples[i] + dc_remove * 0.99;
-			dsamples[i] = d - dc_remove;
-			dc_remove = d;
-		}
-	}
-	// For FM, filter the audio
-	if (rxMode == 5 && fmFilterSize) {
-		www = tan(M_PI * FM_FILTER_DEMPH / filter_srate);	// filter_srate might change
-		nnn = 1.0 / (1.0 + www);
-		a_0 = www * nnn;
-		a_1 = a_0;
-		b_1 = nnn * (www - 1.0);
-		for (i = 0; i < nSamples; i++) {
-			fmFilterBufI[indexFmFilter] =  dsamples[i];
-			accI = 0;
-			j = indexFmFilter;
-			for (k = 0; k < fmFilterSize; k++) {
-				accI += fmFilterBufI[j] * fmFilterI[k];
-				if (++j >= fmFilterSize)
-					j = 0;
-			}
-			//dsamples[i] = accI;
-			if (++indexFmFilter >= fmFilterSize)
-				indexFmFilter = 0;
 			// FM de-emphasis
-			dsamples[i] = y_1 = accI * a_0 + x_1 * a_1 - y_1 * b_1;
-			x_1 = accI;
+			dsamples[i] = FM_y_1 = di * FM_a_0 + FM_x_1 * FM_a_1 - FM_y_1 * FM_b_1;
+			FM_x_1 = di;
 		}
+		nSamples = quisk_dDecimate(dsamples, nSamples, &filtAudio24p3, 2);
+		nSamples = quisk_dFilter(dsamples, nSamples, &filtAudioFmHp);
+		nSamples = quisk_dInterp2HB45(dsamples, nSamples, &HalfBand6);
+		break;
 	}
-	// Perhaps interpolate the samples back to the play rate
-	if (quisk_sound_state.int_filter_interp > 1) {
-		k = quisk_sound_state.int_filter_interp;
-		// from samples a, b, c  make  a, 0, 0, b, 0, 0, c, 0, 0
-		nSamples *= k;
-		for (i = nSamples - 1; i >= 0; i--) {
-			if (i % k == 0)
-				dsamples[i] = dsamples[i / k] * k;
-			else
-				dsamples[i] = 0;
-		}
-		for (i = 0; i < nSamples; i++) {	// low pass filter
-			interpFilterBuf[indexInterpFilter] =  dsamples[i];
-			accI = 0;
-			j = indexInterpFilter;
-			for (k = 0; k < INTERP_FILTER_TAPS; k++) {
-				accI += interpFilterBuf[j] * interpFilterCoef[k];
-				if (++j >= INTERP_FILTER_TAPS)
-					j = 0;
-			}
-			dsamples[i] = accI;
-			if (++indexInterpFilter >= INTERP_FILTER_TAPS)
-				indexInterpFilter = 0;
-		}
+#if DEBUG
+	for (i = 0; i < nSamples; i++) {
+		d = fabs(dsamples[i]);
+		if (levelC < d)
+			levelC = d;
 	}
-	// Perhaps decimate (lower the sample rate) by an additional fraction
-	if (quisk_sound_state.double_filter_decim != 1.0) {
-		nSamples = fDecimate(dsamples, nSamples, quisk_sound_state.double_filter_decim);
+#endif
+
+	// Perhaps decimate by an additional fraction
+	if (double_filter_decim != 1.0)
+		nSamples = fDecimate(dsamples, nSamples, double_filter_decim);
+#if DEBUG
+	for (i = 0; i < nSamples; i++) {
+		d = fabs(dsamples[i]);
+		if (levelD < d)
+			levelD = d;
 	}
+#endif
+	// Interpolate the samples back to the play rate
+	interp = quisk_sound_state.playback_rate / filter_srate;
+	if (interp > 1) {
+		nSamples = quisk_dInterp2HB45(dsamples, nSamples, &HalfBand7);
+		interp /= 2;
+	}
+	if (interp > 1) {
+		nSamples = quisk_dInterp2HB45(dsamples, nSamples, &HalfBand8);
+		interp /= 2;
+	}
+	if (interp > 1) {
+		nSamples = quisk_dInterp2HB45(dsamples, nSamples, &HalfBand9);
+		interp /= 2;
+	}
+	if (interp != 1)
+		printf ("Failure in quisk.c in integer interpolation\n");
+#if DEBUG
+	for (i = 0; i < nSamples; i++) {
+		d = fabs(dsamples[i]);
+		if (levelE < d)
+			levelE = d;
+	}
+#endif
 	// Find the peak signal amplitude, copy sound to output cSamples
 	agcPeak = 0;
 	for (i = 0; i < nSamples; i++) {
@@ -707,9 +796,14 @@ start_agc:
 			agcGain = 1.0 / agcPeak;
 	}
 #if DEBUG
-	if (printit)
+	if (printit) {
+		d = CLIP32;
 		printf("agcGain %12.1lf  agcMaxGain * agcLevel %.1lf  agcPeak %.6lf agcOffGain %.1lf\n",
 			agcGain, agcMaxGain * agcLevel, agcPeak / 2.e9, agcOffGain);
+		printf ("Levels: %12.8lf  %12.8lf %12.8lf  %12.8lf  %12.8lf\n",
+			levelA/d, levelB/d, levelC/d, levelD/d, levelE/d);
+		levelA = levelB = levelC = levelD = levelE = 0;
+	}
 #endif
 	agc_level = agcGain * audioVolume;
 	if (keyupEnvelope < 1.0) {		// raise volume slowly after the key goes up
@@ -763,13 +857,29 @@ static PyObject * get_overrange(PyObject * self, PyObject * args)
 }
 
 static PyObject * get_filter_rate(PyObject * self, PyObject * args)
-{
+{  // return the filter sample rate as used by quisk_process_samples
+	int rate;
+
 	if (!PyArg_ParseTuple (args, ""))
 		return NULL;
-	if (quisk_sound_state.int_filter_decim)
-		return PyInt_FromLong(quisk_sound_state.sample_rate / quisk_sound_state.int_filter_decim);
-	else
-		return PyInt_FromLong(quisk_sound_state.sample_rate);
+	rate = 24000;
+	switch(rxMode) {
+	case 0:
+	case 1:
+		rate /= 4;	  // CW
+		break;
+	case 2:
+	case 3:
+	case 7:
+	default:
+		rate /= 2;	  // SSB
+		break;
+	case 4:
+		break;		  // AM
+	case 5:
+		break;		  // FM
+	}
+	return PyInt_FromLong(rate);
 }
 
 static PyObject * get_smeter(PyObject * self, PyObject * args)
@@ -1061,6 +1171,8 @@ static PyObject * open_sound(PyObject * self, PyObject * args)
 				&quisk_sound_state.mic_playback_rate
 				))
 		return NULL;
+	if (quisk_sound_state.mic_out_volume > 0.7)	// maximum value must leave headroom for
+		quisk_sound_state.mic_out_volume = 0.7;	//   the amplitude and phase adjustments
 	quisk_sound_state.playback_rate = QuiskGetConfigInt("playback_rate", 48000);
 	quisk_mic_preemphasis = QuiskGetConfigDouble("mic_preemphasis", 0.6);
 	//if (quisk_mic_preemphasis < 0.0 || quisk_mic_preemphasis > 1.0)
@@ -1094,7 +1206,6 @@ static PyObject * change_rate(PyObject * self, PyObject * args)	// Called from G
 	if (!PyArg_ParseTuple (args, "ii", &sample_rate, &average_count))
 		return NULL;
 	quisk_sound_state.sample_rate = sample_rate;
-	quisk_set_decimation();
 	rx_udp_read_blocks = 0;		// re-calculate
 	rx_udp_gain_correct = 0;	// re-calculate
 	Py_INCREF (Py_None);
@@ -1194,42 +1305,6 @@ static PyObject * set_filters(PyObject * self, PyObject * args)
 	return Py_None;
 }
 
-PyObject * set_fm_filters(PyObject * self, PyObject * args)
-{  // Enter the coefficients of the audio filter for FM
-   // Storage space is malloc'd.
-	PyObject * filterI;
-	int i, size;
-	PyObject * obj;
-
-	if (!PyArg_ParseTuple (args, "O", &filterI))
-		return NULL;
-	if (PySequence_Check(filterI) != 1) {
-		PyErr_SetString (PyExc_TypeError, "FM filter is not a sequence");
-		return NULL;
-	}
-
-	if (fmFilterSize) {
-		free (fmFilterI);
-		free (fmFilterBufI);
-		fmFilterSize = 0;
-	}
-	size = PySequence_Size(filterI);
-    if (size) {
-		fmFilterSize = size;
-		i = size * sizeof(double);
-		fmFilterI =    (double *)malloc(i);
-		fmFilterBufI = (double *)malloc(i);
-		memset(fmFilterBufI, 0, i);
-		for (i = 0; i < size; i++) {
-			obj = PySequence_GetItem(filterI, i);
-			fmFilterI[i] = PyFloat_AsDouble(obj);
-			Py_XDECREF(obj);
-		}
-	}
-	Py_INCREF (Py_None);
-	return Py_None;
-}
-
 static PyObject * set_noise_blanker(PyObject * self, PyObject * args)
 {
 	if (!PyArg_ParseTuple (args, "i", &quisk_noise_blanker))
@@ -1265,6 +1340,7 @@ static PyObject * set_sidetone(PyObject * self, PyObject * args)
 
 	if (!PyArg_ParseTuple (args, "iid", &quisk_sidetoneCtrl, &rit_freq, &delay))
 		return NULL;
+	//printf("Sidetone control times 5 = %d\n", quisk_sidetoneCtrl * 5);
 	// Simulate log taper pot
 	sidetoneVolume = (exp(quisk_sidetoneCtrl * 0.006908) - 1) / 1000.0;
 	sidetonePhase = cexp((I * 2.0 * M_PI * abs(rit_freq)) / quisk_sound_state.playback_rate);
@@ -1348,6 +1424,7 @@ static PyObject * get_graph(PyObject * self, PyObject * args)
 		i = (int)((double)quisk_tx_tune_freq * fft_size / sample_rate - d2 + 0.5);
 		break;
 	case 3:		// USB:  bandwidth is above tx frequency
+	case 7:
 		i = (int)((double)quisk_tx_tune_freq * fft_size / sample_rate + 0.5);
 		break;
 	}
@@ -1618,6 +1695,13 @@ static PyObject * dft(PyObject * self, PyObject * args)
 	return Xdft(tuple2, 0, window);
 }
 
+static PyObject * is_key_down(PyObject * self, PyObject * args)
+{
+	if (!PyArg_ParseTuple (args, ""))
+		return NULL;
+	return PyInt_FromLong(quisk_is_key_down());
+}
+
 static PyObject * idft(PyObject * self, PyObject * args)
 {
 	PyObject * tuple2;
@@ -1752,6 +1836,7 @@ static PyMethodDef QuiskMethods[] = {
 	{"add_tone", add_tone, METH_VARARGS, "Add a test tone to the data."},
 	{"dft", dft, METH_VARARGS, "Calculate the discrete Fourier transform."},
 	{"idft", idft, METH_VARARGS, "Calculate the inverse discrete Fourier transform."},
+	{"is_key_down", is_key_down, METH_VARARGS, "Check whether the key is down; return 0 or 1."},
 	{"get_state", get_state, METH_VARARGS, "Return a count of read and write errors."},
 	{"get_graph", get_graph, METH_VARARGS, "Return a tuple of graph data."},
 	{"get_filter", get_filter, METH_VARARGS, "Return the frequency response of the receive filter."},
@@ -1765,7 +1850,6 @@ static PyMethodDef QuiskMethods[] = {
 	{"set_ampl_phase", quisk_set_ampl_phase, METH_VARARGS, "Set the sound card amplitude and phase corrections."},
 	{"set_agc", set_agc, METH_VARARGS, "Set the AGC parameters."},
 	{"set_filters", set_filters, METH_VARARGS, "Set the receive audio I and Q channel filters."},
-	{"set_fm_filters", set_fm_filters, METH_VARARGS, "Set the FM audio filter."},
 	{"set_noise_blanker", set_noise_blanker, METH_VARARGS, "Set the noise blanker level."},
 	{"set_rx_mode", set_rx_mode, METH_VARARGS, "Set the receive mode: CWL, USB, AM, etc."},
 	{"set_spot_level", quisk_set_spot_level, METH_VARARGS, "Set the spot mode: 0, 1, ... or -1 for no spot"},
@@ -1777,6 +1861,7 @@ static PyMethodDef QuiskMethods[] = {
 	{"test_3", test_3, METH_VARARGS, "Test 3 function."},
 	{"set_fdx", set_fdx, METH_VARARGS, "Set full duplex mode; ignore the key status."},
 	{"sound_devices", quisk_sound_devices, METH_VARARGS, "Return a list of available sound device names."},
+	{"sound_errors", quisk_sound_errors, METH_VARARGS, "Return a list of text strings with sound devices and error counts"},
 	{"open_sound", open_sound, METH_VARARGS, "Open the the soundcard device."},
 	{"close_sound", close_sound, METH_VARARGS, "Stop the soundcard and release resources."},
 	{"capt_channels", quisk_capt_channels, METH_VARARGS, "Set the I and Q capture channel numbers"},
@@ -1795,9 +1880,31 @@ static PyMethodDef QuiskMethods[] = {
 
 PyMODINIT_FUNC init_quisk (void)
 {
-	PyObject *m;
+	PyObject * m;
+	PyObject * c_api_object;
+	static void * Quisk_API[] = QUISK_API_INIT;
+
 	m = Py_InitModule ("_quisk", QuiskMethods);
+	if (m == NULL) {
+		printf("Py_InitModule of _quisk failed!\n");
+		return;
+	}
+
 	QuiskError = PyErr_NewException ("quisk.error", NULL, NULL);
 	Py_INCREF (QuiskError);
 	PyModule_AddObject (m, "error", QuiskError);
+
+#if ( (PY_VERSION_HEX <  0x02070000) || ((PY_VERSION_HEX >= 0x03000000) && (PY_VERSION_HEX <  0x03010000)) )
+// Old Python interface using CObject
+       /* Create CObjects for handing _quisk symbols to C extensions in other Python modules. */
+       c_api_object = PyCObject_FromVoidPtr(Quisk_API, NULL);
+       if (c_api_object != NULL)
+         PyModule_AddObject(m, "QUISK_C_API", c_api_object);
+#else
+// New Python interface using Capsule
+       /* Create Capsules for handing _quisk symbols to C extensions in other Python modules. */
+       c_api_object = PyCapsule_New(Quisk_API, "_quisk.QUISK_C_API", NULL);
+       if (c_api_object != NULL)
+         PyModule_AddObject(m, "QUISK_C_API", c_api_object);
+#endif
 }
