@@ -93,6 +93,10 @@ color_test		= 'hot pink'			# color of a button used for test (turn off for tx)
 color_freq		= 'lightcyan1'			# background color of frequency and s-meter
 color_entry		= color_freq			# frequency entry box
 color_disable	= 'white'				# text color for a disabled button
+color_bandwidth = 'lemonchiffon2'		# color for bandwidth display; thanks to WB4JFI
+#color_bandwidth = 'lemonchiffon3'
+
+filter_display = 1	# Display the filter bandwidth on the graph screen; 0 or 1; thanks to WB4JFI
 
 # These are the palettes for the waterfall.  The one used is named waterfallPallette,
 # so to use a different one, overwrite this name in your .quisk_conf.py.
@@ -144,7 +148,10 @@ digipanWaterfallPalette = (
 # play (for a panadapter).
 #
 # For the SDR-IQ the soundcard is not used for capture; it only plays back audio.
-# Playback is always 48 kHz stereo.
+
+# Quisk has a custom decimation scheme for each sample rate.  The allowable sample rates
+# are the four SDR-IQ rates plus 24, 48, 96, 192, 240, 384, 480, and 960 ksps.  Other rates
+# can be added.
 
 # Configuration for soundcard capture and playback
 use_sdriq = 0						# Get ADC samples from SDR-IQ is not used
@@ -170,12 +177,14 @@ tx_channel_delay = -1
 # If you use a soundcard with Ethernet control of the VFO, set these parameters:
 rx_ip = ""							# Receiver IP address for VFO control
 
-# If you use an SDR-IQ for capture, see the sample config file in the sdriq package directory.
+# If you use an SDR-IQ for capture, see the sample config file quisk_conf_sdriq.py.
 
 # For the N2ADR 2010 transceiver described in QEX, and for the improved version HiQSDR,
 # see the sample config file in the hiqsdr package directory, and set these:
 # tx_level sets the transmit level 0 to 255 for each band.  The None band is the default.
-tx_level = {None:120, '60':52}
+tx_level = {None:120, '60':110}
+# Mode DGTL uses this instead of tx_level, so you can reduce the power.
+digital_tx_level = 20
 # If you use the HiQSDR hardware, set these:
 # The HiQSDR_BandDict sets the preselect (4 bits) on the X1 connector.
 HiQSDR_BandDict = {'160':1, '80':2, '40':3, '30':4, '20':5, '15':6, '17':7,
@@ -198,6 +207,7 @@ si570_xtal_freq = 114285000
 # This is the received radio sound playback rate.  The default will
 # be 48 kHz for the SDR-IQ and UDP port samples, and sample_rate for sound
 # card capture.  Set it yourself for other rates or hardware.
+# The playback_rate must be 24000, 48000, 96000 or 192000.
 # playback_rate = 48000
 
 # If you use quisk_hardware_fixed.py, this is the fixed VFO frequency in Hertz
@@ -216,6 +226,19 @@ mouse_tune_method = 0	# The Quisk tune frequency changes and the VFO frequency i
 # This is the CW tone frequency in Hertz
 cwTone = 600
 
+# The "DGTL" mode mostly acts like USB, but it sends received audio to an external
+# program that can decode digital modes, and receives audio to transmit.
+# The only program currently supported is Fldigi.  Set Fldigi to USB, XML-RPC control.
+digital_xmlrpc_url = "http://localhost:7362"	# URL for control by XML-RPC
+# Input audio from an external program for use with mode DGTL.  The input must be
+# stereo at 48000 sps.
+digital_input_name = ""		# device name for transmit audio
+# digital_input_name = 'hw:Loopback,0'
+# Output audio to an external program for use with mode DGTL.  The output is
+# stereo at the same sample rate as the radio sound playback.
+digital_output_name = ""	# device name for received audio
+# digital_output_name = digital_input_name
+
 # If you use the microphone feature, the mic_channel_I and Q are the two capture
 # microphone channels.  Quisk uses a monophonic mic, so audio is taken from the I
 # channel, and the Q channel is (currently) ignored.  It is OK to set the same
@@ -231,16 +254,16 @@ cwTone = 600
 # Microphone capture:
 microphone_name = ""		# Name of microphone capture device (or "hw:1")
 mic_sample_rate = 48000		# Microphone capture sample rate in Hertz, must be 48000 or 8000
-mic_channel_I = 0			# Soundcard index of mic capture audio channel
-mic_channel_Q = 0			# Soundcard index of ignored capture channel
+mic_channel_I = 0		# Soundcard index of mic capture audio channel
+mic_channel_Q = 0		# Soundcard index of ignored capture channel
 # Microphone samples sent to soundcard:
 name_of_mic_play = ""		# Name of play device if mic I/Q is sent to a sound card
 mic_playback_rate = 48000	# Playback rate must be a multiple 1, 2, ... of mic_sample_rate
-mic_play_chan_I = 0			# Soundcard index of mic I play channel
-mic_play_chan_Q = 1			# Soundcard index of mic Q play channel
-mic_out_volume = 1.0		# Microphone output volume (after all processing) as a fraction 0.0 to 1.0
+mic_play_chan_I = 0		# Soundcard index of mic I play channel
+mic_play_chan_Q = 1		# Soundcard index of mic Q play channel
+mic_out_volume = 0.7		# Microphone output volume (after all processing) as a fraction 0.0 to 0.7
 # Microphone samples sent to UDP:
-tx_ip = ""					# Transmit IP address for mic sent to UDP (or "192.168.2.195")
+tx_ip = ""				# Transmit IP address for mic sent to UDP (or "192.168.2.195")
 tx_audio_port = 0			# UDP port for mic samples (or 0x553B)
 # Microphone audio processing:
 # The original audio processing used mic_clip = 4.0; mic_preemphasis = -1.0
@@ -272,12 +295,16 @@ add_fdx_button = 0
 # add_extern_demod = "WFM"
 add_extern_demod = ""
 
-# These are the filter bandwidths for each mode:
-FilterBwCW	= (200, 300, 400, 500, 1000, 3000)
-FilterBwSSB	= (1800, 2000, 2200, 2500, 2800, 3300)
-FilterBwAM	= (4000, 5000, 6000, 7000, 8000, 9000)
+# These are the suppressed carrier frequencies for 60 meters
+freq60 = (5330500, 5346500, 5357000, 5371500, 5403500)
+
+# These are the filter bandwidths for each mode.  Quisk has built-in optimized filters
+# for these values, but you can change them if you want.
+FilterBwCW	= (200, 400, 600, 1000, 1500, 3000)
+FilterBwSSB	= (2000, 2200, 2500, 2800, 3000, 3300)
+FilterBwAM	= (4000, 5000, 6000, 8000, 10000, 9000)
 FilterBwFM	= (8000, 10000, 12000, 15000, 17000, 20000)
-FilterBwIMD	= (1800, 2000, 2200, 2500, 2800, 3300)
+FilterBwIMD	= FilterBwSSB
 FilterBwEXT	= (8000, 10000, 12000, 15000, 17000, 20000)
 
 # This is the data used to draw colored lines on the frequency X axis to
@@ -328,16 +355,16 @@ BandPlan = [
   [ 3890000, Phone],
   [ 4000000, None],
   # 60 meters
-  [ 5330600, Phone],
-  [ 5333400, None],
-  [ 5346600, Phone],
-  [ 5349400, None],
-  [ 5366600, Phone],
-  [ 5369400, None],
-  [ 5371600, Phone],
-  [ 5374400, None],
-  [ 5403600, Phone],
-  [ 5406400, None],
+  [ freq60[0], Phone],
+  [ freq60[0] + 2800, None],
+  [ freq60[1], Phone],
+  [ freq60[1] + 2800, None],
+  [ freq60[2], Phone],
+  [ freq60[2] + 2800, None],
+  [ freq60[3], Phone],
+  [ freq60[3] + 2800, None],
+  [ freq60[4], Phone],
+  [ freq60[4] + 2800, None],
   # 40 meters
   [ 7000000, eCW],
   [ 7025000, CW],
