@@ -14,6 +14,8 @@
  (channel0, channel1), (channel0, channel1), ...
 */
 
+#define CORRECT_PLAY_RATE	1
+
 extern struct sound_conf quisk_sound_state;	// Current sound status
 
 static int is_little_endian;		// Test byte order; is it little-endian?
@@ -53,14 +55,17 @@ int quisk_read_alsa(struct sound_dev * dev, complex * cSamples)
 		break;
 	}
 
-	avail = dev->read_frames;	// size of read request
-	if (avail == 0) {			// non-blocking: read available frames
-		snd_pcm_delay(dev->handle, &avail);
+	snd_pcm_delay(dev->handle, &avail);	// available frames
+	dev->dev_latency = avail;
+	if (dev->read_frames == 0) {		// non-blocking: read available frames
 		if (avail == 0)
 			avail = 32;	// read frames to restart from error
 		if (avail > SAMP_BUFFER_SIZE / dev->num_channels)		// limit read request to buffer size
 			avail = SAMP_BUFFER_SIZE / dev->num_channels;
 	}
+	else {
+		avail = dev->read_frames;	// size of read request
+	}
 	nSamples = 0;
 	switch (dev->sample_bytes) {
 	case 2:
@@ -69,7 +74,7 @@ int quisk_read_alsa(struct sound_dev * dev, complex * cSamples)
 			break;
 		}
 		else if (frames <= 0) {		// error
-			quisk_sound_state.read_error++;
+			dev->dev_error++;
 #if DEBUG_IO
 			QuiskPrintTime("read_alsa: frames < 0", 0);
 #endif
@@ -95,7 +100,7 @@ int quisk_read_alsa(struct sound_dev * dev, complex * cSamples)
 			break;
 		}
 		else if (frames <= 0) {		// error
-			quisk_sound_state.read_error++;
+			dev->dev_error++;
 #if DEBUG_IO
 			QuiskPrintTime("read_alsa: frames < 0", 0);
 #endif
@@ -130,7 +135,7 @@ int quisk_read_alsa(struct sound_dev * dev, complex * cSamples)
 			break;
 		}
 		else if (frames <= 0) {		// error
-			quisk_sound_state.read_error++;
+			dev->dev_error++;
 #if DEBUG_IO
 			QuiskPrintTime("read_alsa: frames < 0", 0);
 #endif
@@ -163,6 +168,9 @@ void quisk_play_alsa(struct sound_dev * playdev, int nSamples,
 	int i, n, index;
 	snd_pcm_sframes_t frames, delay;
 	int ii, qq;
+#if DEBUG_IO
+	static int timer=0;
+#endif
 
 	if (!playdev->handle || nSamples <= 0)
 		return;
@@ -176,8 +184,11 @@ void quisk_play_alsa(struct sound_dev * playdev, int nSamples,
 		snd_pcm_delay(playdev->handle, &delay);	// samples left in play buffer
 		break;
 	case SND_PCM_STATE_PREPARED:
-		//printf("State PREPARED\n");
+#if DEBUG_IO
+		//QuiskPrintTime("play_alsa: State PREPARED", 0);
+#endif
 		snd_pcm_delay(playdev->handle, &delay);
+		delay = 0;
 		break;
 	case SND_PCM_STATE_XRUN:
 #if DEBUG_IO
@@ -185,6 +196,7 @@ void quisk_play_alsa(struct sound_dev * playdev, int nSamples,
 		printf("play_alsa: Play underrun; nSamples %d\n", nSamples);
 #endif
 		quisk_sound_state.underrun_error++;
+		playdev->dev_underrun++;
 		snd_pcm_prepare(playdev->handle);
 		delay = 0;
 		break;
@@ -195,18 +207,41 @@ void quisk_play_alsa(struct sound_dev * playdev, int nSamples,
 		delay = 0;
 		break;
 	}
+	playdev->dev_latency = delay;
 	if (report_latency) {		// Report for main playback device
 		quisk_sound_state.latencyPlay = delay;		// samples left in play buffer
 	}
 	// There will be additional samples available to read in the capture buffer.
-
 	index = 0;
+#if CORRECT_PLAY_RATE
+#if DEBUG_IO
+	timer += nSamples;
+	if (timer > playdev->sample_rate) {
+		timer = 0;
+		printf("Samples new %d old %ld total %ld latency_frames %d\n", nSamples, delay, nSamples + delay, playdev->latency_frames);
+	}
+#endif
+	if (nSamples + delay > playdev->latency_frames * 9 / 10) {
+		nSamples--;
+#if DEBUG_IO
+		printf("Remove a sample nSamples %d  delay %d  total %d\n", nSamples, (int)delay, nSamples + (int)delay);
+#endif
+	}
+	else if(nSamples + delay < playdev->latency_frames * 5 / 10) {
+		cSamples[nSamples] = cSamples[nSamples - 1];
+		nSamples++;
+#if DEBUG_IO
+		printf ("Add a sample nSamples %d  delay %d  total %d\n", nSamples, (int)delay, nSamples + (int)delay);
+#endif
+	}
+#endif
 	if (nSamples + delay > playdev->latency_frames) {
 		index = nSamples + delay - playdev->latency_frames;	// write only the most recent samples
 		quisk_sound_state.write_error++;
+		playdev->dev_error++;
 #if DEBUG_IO
 		QuiskPrintTime("", 0);
-		printf("play_alsa: Discard %d of %d samples at %d delay\n", index, nSamples, (int)delay);
+		printf("play_alsa: Discard %d of %d samples at %ld delay\n", index, nSamples, delay);
 #endif
 	}
 
@@ -224,10 +259,14 @@ void quisk_play_alsa(struct sound_dev * playdev, int nSamples,
 #if DEBUG_IO
 				QuiskPrintTime("play_alsa: frames < 0", 0);
 #endif
-				if (frames == -EPIPE)	// underrun
+				if (frames == -EPIPE) {	// underrun
 					quisk_sound_state.underrun_error++;
-				else
+					playdev->dev_underrun++;
+				}
+				else {
 					quisk_sound_state.write_error++;
+					playdev->dev_error++;
+				}
 				snd_pcm_prepare(playdev->handle);
 				frames = snd_pcm_writei (playdev->handle, buffer2, n);
 				if (frames <= 0) {
@@ -266,10 +305,14 @@ void quisk_play_alsa(struct sound_dev * playdev, int nSamples,
 #if DEBUG_IO
 				QuiskPrintTime("play_alsa: frames < 0", 0);
 #endif
-				if (frames == -EPIPE)	// underrun
+				if (frames == -EPIPE) {	// underrun
 					quisk_sound_state.underrun_error++;
-				else
+					playdev->dev_underrun++;
+				}
+				else {
 					quisk_sound_state.write_error++;
+					playdev->dev_error++;
+				}
 				snd_pcm_prepare(playdev->handle);
 				frames = snd_pcm_writei (playdev->handle, buffer3, n);
 				if (frames <= 0) {
@@ -298,10 +341,14 @@ void quisk_play_alsa(struct sound_dev * playdev, int nSamples,
 #if DEBUG_IO
 				QuiskPrintTime("play_alsa: frames < 0", 0);
 #endif
-				if (frames == -EPIPE)	// underrun
+				if (frames == -EPIPE) {	// underrun
 					quisk_sound_state.underrun_error++;
-				else
+					playdev->dev_underrun++;
+				}
+				else {
 					quisk_sound_state.write_error++;
+					playdev->dev_error++;
+				}
 				snd_pcm_prepare(playdev->handle);
 				frames = snd_pcm_writei (playdev->handle, buffer4, n);
 				if (frames <= 0) {
@@ -715,7 +762,7 @@ static int quisk_open_alsa_playback(struct sound_dev * dev)
 #if DEBUG_IO
 	printf("Audio rate %d latency_frames %d\n", sample_rate, dev->latency_frames);
 #endif
-	if (snd_pcm_sw_params_set_start_threshold (handle, sware, dev->latency_frames * 7 / 8) < 0) {
+	if (snd_pcm_sw_params_set_start_threshold (handle, sware, dev->latency_frames * 7 / 10) < 0) {
 		snprintf (quisk_sound_state.err_msg, QUISK_SC_SIZE, "Cannot set start threshold\n");
 		goto errend;
 	}
@@ -754,9 +801,10 @@ errend:
 	return 1;
 }
 
-void quisk_start_sound_alsa (struct sound_dev * pCapture, struct sound_dev * pPlayback,
-		struct sound_dev * pMicCapture, struct sound_dev * pMicPlayback)
+void quisk_start_sound_alsa (struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
 {
+	struct sound_dev * pDev;
+
 	memset(bufferz, 0, sizeof(int) * SAMP_BUFFER_SIZE);
 	is_little_endian = 1;	// Test machine byte order
 	if (*(char *)&is_little_endian == 1)
@@ -765,43 +813,50 @@ void quisk_start_sound_alsa (struct sound_dev * pCapture, struct sound_dev * pPl
 		is_little_endian = 0;
 	if (quisk_sound_state.err_msg[0])
 		return;		// prior error
-	// Open the alsa devices
-	if ( ! pCapture->handle && pCapture->portaudio_index < 0)
-		if (quisk_open_alsa_capture(pCapture))
-			return;		// error
-	if ( ! pPlayback->handle && pPlayback->portaudio_index < 0)
-		if (quisk_open_alsa_playback(pPlayback))
-			return;		// error
-	if ( ! pMicCapture->handle && pMicCapture->portaudio_index < 0)
-		if (quisk_open_alsa_capture(pMicCapture))
-			return;		// error
-	if ( ! pMicPlayback->handle && pMicPlayback->portaudio_index < 0)
-		if (quisk_open_alsa_playback(pMicPlayback))
-			return;		// error
-	if (pCapture->handle && pCapture->portaudio_index < 0)
-		snd_pcm_start((snd_pcm_t *)pCapture->handle);
-	if (pMicCapture->handle && pMicCapture->portaudio_index < 0)
-		snd_pcm_start((snd_pcm_t *)pMicCapture->handle);
+	// Open the alsa playback devices
+	while (1) {
+		pDev = *pPlayback++;
+		if ( ! pDev)
+			break;
+		if ( ! pDev->handle && pDev->portaudio_index < 0)
+			if (quisk_open_alsa_playback(pDev))
+				return;		// error
+	}
+	// Open the alsa capture devices and start them
+	while (1) {
+		pDev = *pCapture++;
+		if ( ! pDev)
+			break;
+		if ( ! pDev->handle && pDev->portaudio_index < 0) {
+			if (quisk_open_alsa_capture(pDev))
+				return;		// error
+			if (pDev->handle)
+				snd_pcm_start((snd_pcm_t *)pDev->handle);
+		}
+	}
 }
 
-void quisk_close_sound_alsa(struct sound_dev * pCapture, struct sound_dev * pPlayback,
-		struct sound_dev * pMicCapture, struct sound_dev * pMicPlayback)
+void quisk_close_sound_alsa(struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
 {
-	if (pCapture->handle && pCapture->portaudio_index < 0) {
-		snd_pcm_drop((snd_pcm_t *)pCapture->handle);
-		snd_pcm_close((snd_pcm_t *)pCapture->handle);
+	struct sound_dev * pDev;
+
+	while (*pCapture) {
+		pDev = *pCapture;
+		if (pDev->handle && pDev->portaudio_index < 0) {
+			snd_pcm_drop((snd_pcm_t *)pDev->handle);
+			snd_pcm_close((snd_pcm_t *)pDev->handle);
+		}
+		pDev->handle = NULL;
+		pCapture++;
 	}
-	if (pPlayback->handle && pPlayback->portaudio_index < 0) {
-		snd_pcm_drop((snd_pcm_t *)pPlayback->handle);
-		snd_pcm_close((snd_pcm_t *)pPlayback->handle);
-	}
-	if (pMicCapture->handle && pMicCapture->portaudio_index < 0) {
-		snd_pcm_drop((snd_pcm_t *)pMicCapture->handle);
-		snd_pcm_close((snd_pcm_t *)pMicCapture->handle);
-	}
-	if (pMicPlayback->handle && pMicPlayback->portaudio_index < 0) {
-		snd_pcm_drop((snd_pcm_t *)pMicPlayback->handle);
-		snd_pcm_close((snd_pcm_t *)pMicPlayback->handle);
+	while (*pPlayback) {
+		pDev = *pPlayback;
+		if (pDev->handle && pDev->portaudio_index < 0) {
+			snd_pcm_drop((snd_pcm_t *)pDev->handle);
+			snd_pcm_close((snd_pcm_t *)pDev->handle);
+		}
+		pDev->handle = NULL;
+		pPlayback++;
 	}
 }
 
