@@ -226,32 +226,47 @@ PyObject * quisk_sound_devices(PyObject * self, PyObject * args)
 	return pylist;
 }
 
-void quisk_start_sound_alsa (struct sound_dev * pCapture, struct sound_dev * pPlayback,
-		struct sound_dev * pMicCapture, struct sound_dev * pMicPlayback)
-{	// DirectX must open the playback device before the (same) capture device
+void quisk_start_sound_alsa (struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
+{
+    struct sound_dev * pDev;
+
 	if (quisk_sound_state.err_msg[0])
 		return;		// prior error
-	if (quisk_open_playback(pPlayback))
-		return;		// error
-	if (quisk_open_playback(pMicPlayback))
-		return;
-	if (quisk_open_capture(pCapture))
-		return;
-	if (quisk_open_capture(pMicCapture))
-		return;
+	// DirectX must open the playback device before the (same) capture device
+	while (1) {
+		pDev = *pPlayback++;
+		if ( ! pDev)
+			break;
+		if (quisk_open_playback(pDev))
+			return;		// error
+	}
+	while (1) {
+		pDev = *pCapture++;
+		if ( ! pDev)
+			break;
+		if (quisk_open_capture(pDev))
+			return;		// error
+	}
 }
 
-void quisk_close_sound_alsa(struct sound_dev * pCapture, struct sound_dev * pPlayback,
-		struct sound_dev * pMicCapture, struct sound_dev * pMicPlayback)
+void quisk_close_sound_alsa(struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
 {
-	if (pCapture->buffer)
-		IDirectSoundCaptureBuffer_Stop((LPDIRECTSOUNDCAPTUREBUFFER)pCapture->buffer);
-	if (pPlayback->buffer)
-		IDirectSoundBuffer8_Stop((LPDIRECTSOUNDBUFFER)pPlayback->buffer);
-	if (pMicCapture->buffer)
-		IDirectSoundCaptureBuffer_Stop((LPDIRECTSOUNDCAPTUREBUFFER)pMicCapture->buffer);
-	if (pMicPlayback->buffer)
-		IDirectSoundBuffer8_Stop((LPDIRECTSOUNDBUFFER)pMicPlayback->buffer);
+	struct sound_dev * pDev;
+
+	while (*pCapture) {
+		pDev = *pCapture;
+		if (pDev->buffer)
+			IDirectSoundCaptureBuffer_Stop((LPDIRECTSOUNDCAPTUREBUFFER)pDev->buffer);
+		pDev->handle = NULL;
+		pCapture++;
+	}
+	while (*pPlayback) {
+		pDev = *pPlayback;
+		if (pDev->buffer)
+            IDirectSoundBuffer8_Stop((LPDIRECTSOUNDBUFFER)pDev->buffer);
+		pDev->handle = NULL;
+		pPlayback++;
+	}
 }
 
 
@@ -278,7 +293,7 @@ int  quisk_read_alsa(struct sound_dev * dev, complex * cSamples)
 #if DEBUG_IO
 		printf ("Get CurrentPosition error 0x%lX\n", hr);
 #endif
-		quisk_sound_state.read_error++;
+		dev->dev_error++;
 		return 0;
 	}
 // printf("dataPos %d\n", dev->dataPos);
@@ -303,7 +318,7 @@ int  quisk_read_alsa(struct sound_dev * dev, complex * cSamples)
 #if DEBUG_IO
 			printf ("Get CurrentPosition two error 0x%lX\n", hr);
 #endif
-			quisk_sound_state.read_error++;
+			dev->dev_error++;
 			return 0;
 		}
 		if (readPos >= dev->dataPos)
@@ -312,6 +327,7 @@ int  quisk_read_alsa(struct sound_dev * dev, complex * cSamples)
 			bytes = readPos - dev->dataPos + dev->play_buf_size;
 	}
 	frames = bytes / bytes_per_frame;	// frames available to read
+	dev->dev_latency = frames;
 	bytes = frames * bytes_per_frame;	// round to frames
 	if ( ! bytes) {
 		return 0;
@@ -322,7 +338,7 @@ int  quisk_read_alsa(struct sound_dev * dev, complex * cSamples)
 		frames = bytes / bytes_per_frame;
 	}
 	if (IDirectSoundCaptureBuffer8_Lock(ptBuf, dev->dataPos, bytes, &pt1, &n1, &pt2, &n2, 0) != DS_OK) {
-		quisk_sound_state.read_error++;
+		dev->dev_error++;
 #if DEBUG_IO
 		printf ("DirecctX capture lock error bytes %d\n", bytes);
 #endif
@@ -417,6 +433,7 @@ void quisk_play_alsa(struct sound_dev * dev, int nSamples,
 	DWORD playPos, writePos;	// hardware index into buffer
 	LPVOID pt1, pt2;
 	DWORD n1, n2;
+	int delay;
 	short * pts;
 	float * ptf;
 	int   * ptl;	// int must be 32 bits
@@ -437,6 +454,7 @@ void quisk_play_alsa(struct sound_dev * dev, int nSamples,
 			printf ("Bad GetCurrentPosition\n");
 #endif
 			quisk_sound_state.write_error++;
+			dev->dev_error++;
 			playPos = writePos = 0;
 		}
 #if DEBUG_IO
@@ -446,14 +464,29 @@ void quisk_play_alsa(struct sound_dev * dev, int nSamples,
 				(int)playPos, (int)writePos, (int)(writePos - playPos) / bytes_per_frame);
 		}
 #endif
-		if (report_latency) {			// Report latency for main playback device
-			if (dev->dataPos >= playPos)
-				count = dev->dataPos - playPos;
-			else
-				count = dev->dataPos - playPos + dev->play_buf_size;
-			frames = count / bytes_per_frame;	// frames in play buffer
-			quisk_sound_state.latencyPlay = frames;
+		if (dev->dataPos >= playPos)
+			count = dev->dataPos - playPos;
+		else
+			count = dev->dataPos - playPos + dev->play_buf_size;
+		delay = count / bytes_per_frame;	// frames in play buffer
+		dev->dev_latency = delay;
+		if (report_latency)			// Report latency for main playback device
+			quisk_sound_state.latencyPlay = delay;
+#if 0
+		if (nSamples + delay > dev->latency_frames * 9 / 10) {
+			nSamples--;
+#if DEBUG_IO
+			printf("Remove a sample nSamples %d  delay %d  total %d\n", nSamples, (int)delay, nSamples + (int)delay);
+#endif
 		}
+		else if(nSamples + delay < dev->latency_frames * 5 / 10) {
+			cSamples[nSamples] = cSamples[nSamples - 1];
+			nSamples++;
+#if DEBUG_IO
+			printf ("Add a sample nSamples %d  delay %d  total %d\n", nSamples, (int)delay, nSamples + (int)delay);
+#endif
+		}
+#endif
 		// Measure writePos to dataPos, the space available to write samples
 		if (dev->dataPos >= writePos)
 			count = dev->dataPos - writePos;
@@ -463,6 +496,7 @@ void quisk_play_alsa(struct sound_dev * dev, int nSamples,
 		// Check for underrun as indicated by excessive samples in two-second buffer
 		if (frames > dev->sample_rate) {	// more than one second of samples
 			quisk_sound_state.underrun_error++;
+			dev->dev_underrun++;
 			// write samples at write pointer + latency
 			dev->dataPos = writePos + (dev->latency_frames - nSamples) * bytes_per_frame;
 			dev->dataPos = dev->dataPos % dev->play_buf_size;
@@ -473,6 +507,7 @@ void quisk_play_alsa(struct sound_dev * dev, int nSamples,
 		// Check if play buffer is too full
 		else if (frames + nSamples > dev->latency_frames * 15 / 10) {
 			quisk_sound_state.write_error++;
+			dev->dev_error++;
 			// write samples at write pointer + latency
 			dev->dataPos = writePos + (dev->latency_frames - nSamples) * bytes_per_frame;
 			dev->dataPos = dev->dataPos % dev->play_buf_size;
@@ -488,6 +523,7 @@ void quisk_play_alsa(struct sound_dev * dev, int nSamples,
 		printf ("DirectX play lock error\n");
 #endif
 		quisk_sound_state.write_error++;
+		dev->dev_error++;
 		return;
 	}
 	dev->dataPos += bytes;	// update data write position
@@ -565,7 +601,7 @@ void quisk_play_portaudio(struct sound_dev * dev, int j, complex * samp, int i)
 {
 }
 
-void quisk_start_sound_portaudio(struct sound_dev * dev, struct sound_dev * dev2, struct sound_dev * dev3, struct sound_dev * dev4)
+void quisk_start_sound_portaudio(struct sound_dev ** pCapture, struct sound_dev ** pPlayback)
 {
 }
 
